# Generated by Django 3.1.6 on 2021-02-22 12:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("opcalendar", "0020_auto_20210125_1007"),
    ]

    operations = [
        migrations.AddField(
            model_name="eventhost",
            name="logo_url",
            field=models.CharField(blank=True, max_length=150),
        ),
    ]
