# Generated by Django 3.1.2 on 2020-11-04 12:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("opcalendar", "0005_auto_20201104_1127"),
    ]

    operations = [
        migrations.AddField(
            model_name="eventhost",
            name="details",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="discord",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="facebook",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="fleet_comms",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="fleet_doctrines",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="ingame_channel",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="ingame_mailing_list",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="twitch",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="website",
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="youtube",
            field=models.CharField(blank=True, max_length=150),
        ),
    ]
