# Generated by Django 3.1.2 on 2021-01-13 08:28

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("opcalendar", "0012_auto_20210112_0934"),
    ]

    operations = [
        migrations.AddField(
            model_name="eventimport",
            name="creator",
            field=models.ForeignKey(
                default="1",
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]
