# Generated by Django 3.1.2 on 2021-01-13 08:36

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("opcalendar", "0013_eventimport_creator"),
    ]

    operations = [
        migrations.AlterField(
            model_name="eventimport",
            name="creator",
            field=models.ForeignKey(
                default="1",
                help_text="User that has been used to create the fleet (most often the superuser who manages the plugin)",
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="eventimport",
            name="host",
            field=models.ForeignKey(
                default=1,
                help_text="The AA host that will be used for the pulled events",
                on_delete=django.db.models.deletion.CASCADE,
                to="opcalendar.eventhost",
            ),
        ),
        migrations.AlterField(
            model_name="eventimport",
            name="operation_type",
            field=models.ForeignKey(
                help_text="Operation type and ticker that will be assigned for the pulled fleets",
                on_delete=django.db.models.deletion.CASCADE,
                to="opcalendar.eventcategory",
            ),
        ),
        migrations.AlterField(
            model_name="eventimport",
            name="source",
            field=models.CharField(
                choices=[
                    ("Spectre Fleet", "Spectre Fleet"),
                    ("EVE University", "EVE University"),
                ],
                help_text="The API source where you want to pull events from",
                max_length=32,
            ),
        ),
    ]
