# Generated by Django 3.1.2 on 2020-11-04 11:26

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("opcalendar", "0003_eventhost"),
    ]

    operations = [
        migrations.AddField(
            model_name="event",
            name="host",
            field=models.ForeignKey(
                default=1,
                on_delete=django.db.models.deletion.CASCADE,
                to="opcalendar.eventhost",
            ),
        ),
    ]
