# Generated by Django 3.2.10 on 2022-01-15 11:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("opcalendar", "0024_auto_20210429_1111"),
    ]

    operations = [
        migrations.AddField(
            model_name="event",
            name="repeat_event",
            field=models.CharField(
                blank=True,
                choices=[("DD", "Daily"), ("MM", "Monthly"), ("YY", "Yearly")],
                default=False,
                help_text="Select if you want to repeat this event in the future",
                max_length=32,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="event",
            name="repeat_times",
            field=models.IntegerField(
                blank=True,
                default=False,
                help_text="How many times do you want to repeat this event?",
                null=True,
            ),
        ),
    ]
