{% extends 'opcalendar/base.html' %}

{% block title %} Signup {% endblock %}

{% block content %}
<div class="row">
    <div class="col-md-3"></div>
    <div class="col-md-6  m-2">
        <form action="" method="post">
            {% csrf_token %}
            {{ form }}
            <input class="btn btn-primary mt-2" type="submit" value="Signup">
        </form>
    </div>
</div>
{% endblock %}
