# Generated by Django 3.1.6 on 2021-02-22 12:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("opcalendar", "0021_eventhost_logo_url"),
    ]

    operations = [
        migrations.AlterField(
            model_name="eventhost",
            name="community",
            field=models.CharField(help_text="Name of the community", max_length=150),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="details",
            field=models.CharField(
                blank=True,
                help_text="Short description about the host.",
                max_length=150,
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="discord",
            field=models.CharField(
                blank=True, help_text="Discord link URL", max_length=150
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="facebook",
            field=models.CharField(
                blank=True, help_text="Facebook link URL", max_length=150
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="fleet_comms",
            field=models.CharField(
                blank=True,
                help_text="Link or description for primary comms such as discord link",
                max_length=150,
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="fleet_doctrines",
            field=models.CharField(
                blank=True,
                help_text="Link or description to the doctrines",
                max_length=150,
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="ingame_channel",
            field=models.CharField(
                blank=True, help_text="Ingame channel name", max_length=150
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="logo_url",
            field=models.CharField(
                blank=True,
                help_text="Absolute URL for the community logo",
                max_length=150,
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="twitch",
            field=models.CharField(
                blank=True, help_text="Twitch link URL", max_length=150
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="twitter",
            field=models.CharField(
                blank=True, help_text="Twitter link URL", max_length=150
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="youtube",
            field=models.CharField(
                blank=True, help_text="Youtube link URL", max_length=150
            ),
        ),
    ]
