{% extends 'opcalendar/base.html' %}

{% block title %}
Add EventMember
{% endblock %}

{% block content %}
<div class="clearfix">
  <a class="btn btn-info left" href="{% url 'opcalendar:calendar' %}"> Calendar </a>
</div>

<form method="post">
  {% csrf_token %}
  <table class="form form-table">
    {{ form }}
    <tr><td colspan="2"><button type="submit" class="btn btn-info right"> Submit </button></td></tr>
  </table>
</form>
{% endblock %}
